@@ -16,7 +16,7 @@
 use crate::{
     component::COMPONENT_PHP_ID,
     context::RequestContext,
-    module::SKYWALKING_VERSION,
+    module::{is_enable, SKYWALKING_VERSION},
     util::{catch_unwind_result, get_sapi_module_name, z_val_to_string},
 };
 use anyhow::{anyhow, Context};
@@ -25,15 +25,18 @@ use once_cell::sync::Lazy;
 use phper::{arrays::ZArr, eg, pg, sg, sys, values::ZVal};
 use skywalking::trace::{propagation::decoder::decode_propagation, tracer};
 use std::{
-    convert::Infallible,
     panic::AssertUnwindSafe,
     ptr::null_mut,
     sync::atomic::{AtomicBool, AtomicPtr, Ordering},
 };
 use tracing::{error, instrument, trace, warn};
+use url::Url;
 
 #[instrument(skip_all)]
 pub fn init() {
+    if !is_enable() {
+        return;
+    }
     if get_sapi_module_name().to_bytes() == b"fpm-fcgi" {
         if let Err(err) = catch_unwind_result(request_init_for_fpm) {
             error!(mode = "fpm", ?err, "request init failed");
@@ -43,6 +46,9 @@ pub fn init() {
 
 #[instrument(skip_all)]
 pub fn shutdown() {
+    if !is_enable() {
+        return;
+    }
     if get_sapi_module_name().to_bytes() == b"fpm-fcgi" {
         if let Err(err) = catch_unwind_result(request_shutdown_for_fpm) {
             error!(mode = "fpm", ?err, "request shutdown failed");
@@ -56,10 +62,10 @@ fn request_init_for_fpm() -> crate::Result<()> {
     let server = get_page_request_server()?;
 
     let header = get_page_request_header(server);
-    let uri = get_page_request_uri(server);
+    let url = get_page_request_url(server)?;
     let method = get_page_request_method(server);
 
-    create_request_context(None, header.as_deref(), &method, &uri)
+    create_request_context(None, header.as_deref(), &method, &url)
 }
 
 fn request_shutdown_for_fpm() -> crate::Result<()> {
@@ -91,13 +97,34 @@ fn get_page_request_header(server: &ZArr) -> Option<String> {
     }
 }
 
-fn get_page_request_uri(server: &ZArr) -> String {
-    server
+fn get_page_request_url(server: &ZArr) -> crate::Result<Url> {
+    let scheme = if [Some("1"), Some("on")]
+        .contains(&server.get("HTTPS").and_then(z_val_to_string).as_deref())
+    {
+        "https"
+    } else {
+        "http"
+    };
+
+    let addr = server
+        .get("HTTP_HOST")
+        .and_then(z_val_to_string)
+        .or_else(|| {
+            server
+                .get("SERVER_PORT")
+                .and_then(z_val_to_string)
+                .map(|port| format!("localhost:{}", port))
+        })
+        .unwrap_or_else(|| "unknown:0".to_string());
+
+    let uri = server
         .get("REQUEST_URI")
         .and_then(z_val_to_string)
         .or_else(|| server.get("PHP_SELF").and_then(z_val_to_string))
         .or_else(|| server.get("SCRIPT_NAME").and_then(z_val_to_string))
-        .unwrap_or_else(|| "{unknown}".to_string())
+        .unwrap_or_else(|| "/unknown".to_string());
+
+    Ok(Url::parse(&format!("{}://{}{}", scheme, addr, uri))?)
 }
 
 fn get_page_request_method(server: &ZArr) -> String {
@@ -118,6 +145,9 @@ fn get_page_request_server<'a>() -> anyhow::Result<&'a ZArr> {
     }
 }
 
+pub const HACK_SWOOLE_ON_REQUEST_FUNCTION_NAME: &str =
+    "skywalking_hack_swoole_on_request_please_do_not_use";
+
 /// Hold the response fd and status code kvs, because I dont't found that
 /// response has the status field, so I hook the response.status method, maybe
 /// there is a better way?
@@ -129,11 +159,11 @@ pub static IS_SWOOLE: AtomicBool = AtomicBool::new(false);
 
 /// The function is used by swoole plugin, to surround the callback of on
 /// request.
-pub fn skywalking_hack_swoole_on_request(args: &mut [ZVal]) -> Result<(), Infallible> {
+pub fn skywalking_hack_swoole_on_request(args: &mut [ZVal]) -> phper::Result<ZVal> {
     let f = ORI_SWOOLE_ON_REQUEST.load(Ordering::Relaxed);
     if f.is_null() {
         error!("Origin swoole on request handler is null");
-        return Ok(());
+        return Ok(ZVal::from(()));
     }
     let f = unsafe { ZVal::from_mut_ptr(f) };
 
@@ -142,7 +172,8 @@ pub fn skywalking_hack_swoole_on_request(args: &mut [ZVal]) -> Result<(), Infall
         error!(mode = "swoole", ?err, "request init failed");
     }
 
-    if let Err(err) = f.call(&mut *args) {
+    let return_value = f.call(&mut *args);
+    if let Err(err) = &return_value {
         error!(
             mode = "swoole",
             ?err,
@@ -158,36 +189,33 @@ pub fn skywalking_hack_swoole_on_request(args: &mut [ZVal]) -> Result<(), Infall
         }
     }
 
-    Ok(())
+    return_value
 }
 
 fn request_init_for_swoole(request: &mut ZVal) -> crate::Result<()> {
-    let request = request
-        .as_mut_z_obj()
-        .context("swoole request isn't object")?;
+    let request = request.as_z_obj().context("swoole request isn't object")?;
 
     let fd = request
-        .get_mut_property("fd")
+        .get_property("fd")
         .as_long()
         .context("swoole request fd not exists")?;
 
-    let header = {
-        let headers = request
-            .get_mut_property("header")
-            .as_z_arr()
-            .context("swoole request header not exists")?;
-        get_swoole_request_header(headers)
-    };
+    let headers = request
+        .get_property("header")
+        .as_z_arr()
+        .context("swoole request header not exists")?;
+
+    let header = get_swoole_request_header(headers);
 
     let server = request
-        .get_mut_property("server")
+        .get_property("server")
         .as_z_arr()
         .context("swoole request server not exists")?;
 
-    let uri = get_swoole_request_uri(server);
     let method = get_swoole_request_method(server);
+    let url = get_swoole_request_url(server, headers)?;
 
-    create_request_context(Some(fd), header.as_deref(), &method, &uri)
+    create_request_context(Some(fd), header.as_deref(), &method, &url)
 }
 
 fn request_shutdown_for_swoole(response: &mut ZVal) -> crate::Result<()> {
@@ -221,11 +249,30 @@ fn get_swoole_request_header(header: &ZArr) -> Option<String> {
     }
 }
 
-fn get_swoole_request_uri(server: &ZArr) -> String {
-    server
+fn get_swoole_request_url(server: &ZArr, headers: &ZArr) -> crate::Result<Url> {
+    let addr = headers
+        .get("host")
+        .and_then(z_val_to_string)
+        .or_else(|| {
+            server
+                .get("server_port")
+                .and_then(z_val_to_string)
+                .map(|port| format!("localhost:{}", port))
+        })
+        .unwrap_or_else(|| "unknown:0".to_string());
+
+    let uri = server
         .get("request_uri")
         .and_then(z_val_to_string)
-        .unwrap_or_else(|| "{unknown}".to_string())
+        .unwrap_or_else(|| "/unknown".to_string());
+
+    let query = server
+        .get("query_string")
+        .and_then(z_val_to_string)
+        .map(|s| format!("?{}", s))
+        .unwrap_or_default();
+
+    Ok(Url::parse(&format!("http://{}{}{}", addr, uri, query))?)
 }
 
 fn get_swoole_request_method(server: &ZArr) -> String {
@@ -236,7 +283,7 @@ fn get_swoole_request_method(server: &ZArr) -> String {
 }
 
 fn create_request_context(
-    request_id: Option<i64>, header: Option<&str>, method: &str, uri: &str,
+    request_id: Option<i64>, header: Option<&str>, method: &str, url: &Url,
 ) -> crate::Result<()> {
     let propagation = header
         .map(decode_propagation)
@@ -247,16 +294,17 @@ fn create_request_context(
 
     let mut ctx = tracer::create_trace_context();
 
-    let operation_name = format!("{method}:{uri}");
+    let operation_name = format!("{}:{}", method, url.path());
     let mut span = match propagation {
         Some(propagation) => ctx.create_entry_span_with_propagation(&operation_name, &propagation),
         None => ctx.create_entry_span(&operation_name),
     };
-    span.with_span_object_mut(|span| {
-        span.component_id = COMPONENT_PHP_ID;
-        span.add_tag("url", uri);
-        span.add_tag("http.method", method);
-    });
+
+    let mut span_object = span.span_object_mut();
+    span_object.component_id = COMPONENT_PHP_ID;
+    span_object.add_tag("url", url.to_string());
+    span_object.add_tag("http.method", method);
+    drop(span_object);
 
     RequestContext::set_global(
         request_id,
@@ -277,7 +325,7 @@ fn finish_request_context(request_id: Option<i64>, status_code: i32) -> crate::R
 
     entry_span.add_tag("http.status_code", &status_code.to_string());
     if status_code >= 400 {
-        entry_span.with_span_object_mut(|span| span.is_error = true);
+        entry_span.span_object_mut().is_error = true;
     }
 
     drop(entry_span);
