@@ -17,6 +17,7 @@ mod common;
 
 use crate::common::{
     COLLECTOR_HTTP_ADDRESS, HTTP_CLIENT, PROXY_SERVER_1_ADDRESS, SWOOLE_SERVER_1_ADDRESS,
+    SWOOLE_SERVER_2_ADDRESS,
 };
 use reqwest::{header::CONTENT_TYPE, RequestBuilder, StatusCode};
 use std::{
@@ -31,7 +32,7 @@ async fn e2e() {
     let fixture = common::setup().await;
 
     // TODO Prefer to listen the server ready signal.
-    sleep(Duration::from_secs(3)).await;
+    sleep(Duration::from_secs(5)).await;
 
     let result = catch_unwind(|| {
         task::block_in_place(|| {
@@ -48,12 +49,20 @@ async fn e2e() {
 
 async fn run_e2e() {
     request_fpm_curl().await;
+    request_fpm_curl_multi().await;
     request_fpm_pdo().await;
     request_fpm_predis().await;
     request_fpm_mysqli().await;
     request_fpm_memcached().await;
     request_fpm_redis().await;
+    request_fpm_rabbitmq().await;
     request_swoole_curl().await;
+    request_swoole_2_curl().await;
+    request_swoole_2_pdo().await;
+    request_swoole_2_mysqli().await;
+    request_swoole_2_memcached().await;
+    request_swoole_2_redis().await;
+    request_swoole_2_predis().await;
     sleep(Duration::from_secs(3)).await;
     request_collector_validate().await;
 }
@@ -66,6 +75,17 @@ async fn request_fpm_curl() {
     .await;
 }
 
+async fn request_fpm_curl_multi() {
+    request_common(
+        HTTP_CLIENT.get(format!(
+            "http://{}/curl-multi.enter.php",
+            PROXY_SERVER_1_ADDRESS
+        )),
+        "ok",
+    )
+    .await;
+}
+
 async fn request_fpm_pdo() {
     request_common(
         HTTP_CLIENT.get(format!("http://{}/pdo.php", PROXY_SERVER_1_ADDRESS)),
@@ -112,6 +132,14 @@ async fn request_fpm_redis() {
     .await;
 }
 
+async fn request_fpm_rabbitmq() {
+    request_common(
+        HTTP_CLIENT.get(format!("http://{}/rabbitmq.php", PROXY_SERVER_1_ADDRESS)),
+        "ok",
+    )
+    .await;
+}
+
 async fn request_swoole_curl() {
     request_common(
         HTTP_CLIENT.get(format!("http://{}/curl", SWOOLE_SERVER_1_ADDRESS)),
@@ -120,6 +148,54 @@ async fn request_swoole_curl() {
     .await;
 }
 
+async fn request_swoole_2_curl() {
+    request_common(
+        HTTP_CLIENT.get(format!("http://{}/curl", SWOOLE_SERVER_2_ADDRESS)),
+        "ok",
+    )
+    .await;
+}
+
+async fn request_swoole_2_pdo() {
+    request_common(
+        HTTP_CLIENT.get(format!("http://{}/pdo", SWOOLE_SERVER_2_ADDRESS)),
+        "ok",
+    )
+    .await;
+}
+
+async fn request_swoole_2_mysqli() {
+    request_common(
+        HTTP_CLIENT.get(format!("http://{}/mysqli", SWOOLE_SERVER_2_ADDRESS)),
+        "ok",
+    )
+    .await;
+}
+
+async fn request_swoole_2_memcached() {
+    request_common(
+        HTTP_CLIENT.get(format!("http://{}/memcached", SWOOLE_SERVER_2_ADDRESS)),
+        "ok",
+    )
+    .await;
+}
+
+async fn request_swoole_2_redis() {
+    request_common(
+        HTTP_CLIENT.get(format!("http://{}/redis", SWOOLE_SERVER_2_ADDRESS)),
+        "ok",
+    )
+    .await;
+}
+
+async fn request_swoole_2_predis() {
+    request_common(
+        HTTP_CLIENT.get(format!("http://{}/predis", SWOOLE_SERVER_2_ADDRESS)),
+        "ok",
+    )
+    .await;
+}
+
 async fn request_collector_validate() {
     request_common(
         HTTP_CLIENT
